@@ -65,7 +65,7 @@ using namespace std;
 #define POOLMAXAGE 50 // seconds
 
 
-const time_t cleaner::never=ULONG_MAX;
+//const time_t cleaner::never=ULONG_MAX;
 
 // some globals, set only once
 static struct stat statTempl;
@@ -221,7 +221,7 @@ public:
 			}
 #define SETERROR { nErr=__LINE__; return false;}
 			bool &m_isFirst;
-			bool DownloadStartedStoreHeader(const header &head, const char*)
+			SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &head, const char*, bool bRestarted)
 			{
 				_cerr(head.frontLine<<endl);
 				m_head = head; // XXX: bloat, only status line and contlen required
@@ -230,6 +230,11 @@ public:
 				if(st == 416)
 					return true; // EOF
 
+				if(bRestarted) // throw the head away, the data should be ok
+				{
+					return true; // XXX, add more checks?
+				}
+
 				if(st != 200 && st != 206)
 				{
 					SETERROR;
@@ -304,7 +309,7 @@ public:
 		bIsFirst=false;
 
 
-		if (m_ftype == FILE_PKG && fidOrig != fid)
+		if (m_ftype == rechecks::FILE_PKG && fidOrig != fid)
 		{
 			lockguard g(mxCache);
 			remote_info_cache[m_path] = fid;
@@ -358,8 +363,11 @@ public:
 			{
 				m_bHeadOnly = true;
 			}
-			bool DownloadStartedStoreHeader(const header &head, const char*)
+			SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &head, const char*, bool bRestart)
 			{
+				if(bRestart)
+					return true;
+
 				m_head = head; // XXX: bloat, only status line and contlen required
 				m_status = FIST_COMPLETE;
 				return true;
@@ -378,7 +386,7 @@ public:
 			return -EIO;
 		else if (nHttpCode == 200)
 		{
-			if (m_ftype == FILE_PKG) // not caching volatile stuff
+			if (m_ftype == rechecks::FILE_PKG) // not caching volatile stuff
 			{
 				lockguard g(mxCache);
 				remote_info_cache[m_path] =
@@ -565,9 +573,9 @@ static int acngfs_getattr(const char *path, struct stat *stbuf)
 	if(!path)
 		return -1;
 
-	UINT type = rechecks::GetFiletype(path);
+	rechecks::eMatchType type = rechecks::GetFiletype(path);
 	_cerr( "type: " << type);
-	if (type == FILE_PKG || type == FILE_INDEX)
+	if (type == rechecks::FILE_PKG || type == rechecks::FILE_INDEX)
 	{
 		if(0 == tDlDescLocal(path, type).Stat(*stbuf))
 			return 0;
@@ -691,12 +699,12 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 
 	tDlDesc *p(NULL);
 	struct stat stbuf;
-	UINT ftype = rechecks::GetFiletype(path);
+	rechecks::eMatchType ftype = rechecks::GetFiletype(path);
 
 	MYTRY
 	{
 		// ok... if that is a remote object, can we still use local access instead?
-		if(!altPath.empty() && FILE_PKG == ftype)
+		if(!altPath.empty() && rechecks::FILE_PKG == ftype)
 		{
 				p = new tDlDescLocal(path, ftype);
 				if(p)
@@ -718,7 +726,7 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 			return -EIO;
 		}
 	}
-	MYCATCH(std::bad_alloc)
+	MYCATCH(std::bad_alloc&)
 	{
 		return -EIO;
 	}
