@@ -31,7 +31,7 @@ tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx;
 extern int debug, numcores, offlinemode, foreground, verbose, stupidfs, forcemanaged, keepnver,
 verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize, usewrap,
 exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, dirperms, fileperms,
-maxtempdelay, redirmax;
+maxtempdelay, redirmax, vrangeops;
 
 // processed config settings
 extern tHttpUrl proxy_info;
@@ -39,7 +39,7 @@ extern mstring agentheader;
 
 extern int conprotos[2];
 
-bool SetOption(const mstring &line, bool bQuiet=false);
+bool SetOption(const mstring &line, bool bQuiet=false, NoCaseStringMap *pDupeChecker=NULL);
 void ReadConfigDirectory(const char*);
 
 //! Prepare various things resulting from variable combinations, etc.
@@ -85,14 +85,16 @@ mstring EncodeBase64Auth(const cmstring & s);
 
 namespace rechecks
 {
-#define FILE_PKG 0
-#define FILE_INDEX 1
-#define	FILE_INVALID 2
-unsigned int GetFiletype(const mstring &);
-bool MatchWhitelist(const mstring &);
+enum eMatchType
+{
+	FILE_INVALID = -1, FILE_PKG = 0, FILE_INDEX = 1, WHITELIST = 2, NASTY_PATH = 3, PASSTHROUGH = 4,
+	ematchtype_max
+};
+bool Match(cmstring &in, eMatchType type);
+
+eMatchType GetFiletype(const mstring &);
 bool MatchUncacheableRequest(const mstring &);
 bool MatchUncacheableTarget(const mstring &);
-
 bool CompileExpressions();
 }
 
