@@ -10,18 +10,25 @@
 
 #include "lockable.h"
 #include <ctime>
+#include <limits>
 
 class cleaner : public condition
 {
 public:
 	cleaner();
-	void Do();
 	virtual ~cleaner();
-	static const time_t never;
+	void RunLatestAt(time_t when);
 
-protected:
-	bool m_bAssigned;
-	bool m_bAwaken;
+
+	void WorkLoop();
+	void Stop();
+
+	inline static const time_t never() { return MYSTD:: numeric_limits<time_t>::max(); };
+
+private:
+	time_t m_nextRun;
+	pthread_t m_thr;
+	bool m_bTerminating;
 };
 
 extern cleaner g_victor;
