@@ -35,8 +35,9 @@ public:
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow)=0;
 	
 	// downloader instruments
-	
-	virtual bool DownloadStartedStoreHeader(const header & h, const char *pNextData)=0;
+	typedef extended_bool<bool, false> SuccessWithTransErrorFlag;
+	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *pNextData,
+			bool bForcedRestart) =0;
 	void IncDlRefCount();
 	void DecDlRefCount(const mstring & sReasonStatusLine);
 	//virtual void SetFailureMode(const mstring & message, FiStatus fist=FIST_ERROR,
@@ -62,6 +63,9 @@ public:
 
 	bool SetupClean(bool bForce=false);
 	
+	/// mark the item as complete as-is, assuming that sizeseen is correct
+	void SetupComplete();
+
 	uint64_t m_nIncommingCount;
 	off_t m_nSizeSeen;
 	mstring m_sPathAbs, m_sPathRel;
@@ -106,7 +110,8 @@ public:
 	inline fileitem_with_storage(cmstring &s) : fileitem_base(s) {};
 	// send helper like wrapper for sendfile. Just declare virtual here to make it better customizable later.
 	virtual ssize_t SendData(int confd, int filefd, off_t &nSendPos, size_t nMax2SendNow);
-	virtual bool DownloadStartedStoreHeader(const header & h, const char *pNextData);
+	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *pNextData,
+			bool bForcedRestart);
 	virtual bool StoreFileData(const char *data, unsigned int size);
 };
 
