@@ -12,6 +12,8 @@ class header {
          INVALID,
          HEAD,
          GET,
+         POST,
+         CONNECT,
          ANSWER
       };
       enum eHeadPos {
@@ -68,7 +70,11 @@ class header {
       void set(eHeadPos, const char *s, size_t len);
       void set(eHeadPos, off_t nValue);
       void del(eHeadPos);
-      inline int getStatus() const { return frontLine.length()>9 ? atoi(frontLine.c_str()+8) : 500; };
+
+      inline const char * getCodeMessage() const {
+    	  return frontLine.length()>9 ? frontLine.c_str()+9 : "";
+      }
+      inline int getStatus() const { int r=atoi(getCodeMessage()); return r ? r : 500; }
       void clear();
       
       mstring ToString() const;
