@@ -84,7 +84,7 @@ class job {
 
       bool ParseRange();
 
-      off_t m_nRangeFrom, m_nRangeTo;
+      off_t m_nReqRangeFrom, m_nReqRangeTo;
 
       int m_nRedirRemaining;
       bool CreateRedirect(header &respHead);
