@@ -17,6 +17,8 @@
 #include <deque>
 #include <limits>
 #include <cstdio>
+#include <ctime>
+#include <cstring>
 
 #include <fcntl.h>
 #include <pthread.h>
@@ -24,7 +26,6 @@
 #include <stdlib.h>
 #include <errno.h>
 
-
 #define EXTREME_MEMORY_SAVING false
 
 typedef MYSTD::string mstring;
@@ -408,5 +409,33 @@ static inline time_t GetTime()
 	return ::time(0);
 }
 
+static inline UINT FormatTime(char *buf, const time_t cur)
+{
+	struct tm tmp;
+	gmtime_r(&cur, &tmp);
+	asctime_r(&tmp, buf);
+	//memcpy(buf + 24, " GMT", 4); // wrong, only needed for rfc-822 format, not for asctime's
+	//return 28;
+	return 24;
+}
+
+struct tCurrentTime
+{
+	char buf[30];
+	UINT len;
+	inline tCurrentTime() { len=FormatTime(buf, time(NULL)); }
+	inline operator mstring() { return mstring(buf, len); }
+};
+
+// represents a boolean value like a normal bool but also carries additional data
+template <typename Textra, Textra defval>
+struct extended_bool
+{
+	bool value;
+	Textra xdata;
+	inline operator bool() { return value; }
+	inline extended_bool(bool val, Textra xtra = defval) : value(val), xdata(xtra) {};
+};
+
 #endif // _META_H
 
