@@ -5,8 +5,7 @@
 #include "expiration.h"
 
 #include "fileio.h"
-#include <time.h>
-#include <string.h>
+#include <cstring>
 #include "csmapping.h"
 
 
