@@ -24,6 +24,8 @@ using namespace MYSTD;
 #define sProblemLoc szPath<< ':'<< _iterPos(it, lines)
 
 #define BARF(x) { cerr << x << endl; exit(EXIT_FAILURE); }
+#define BADSTUFF_PATTERN "\\.\\.($|%|/)"
+mstring PTHOSTS_PATTERN;
 
 namespace rechecks
 {
@@ -80,6 +82,7 @@ MapNameToString n2sTbl[] = {
 		,{ "DontCacheResolved",	&tmpDontcacheResolved, 0}
 		,{ "PrecacheFor", &mirrorsrcs, 0}
 		,{ "RequestAppendix", &requestapx, 0}
+		,{ "PassThroughPattern", &PTHOSTS_PATTERN, 0}
 };
 
 MapNameToInt n2iTbl[] = {
@@ -108,6 +111,7 @@ MapNameToInt n2iTbl[] = {
 		,{ "UseWrap", &usewrap, NULL, 10}
 		,{ "FreshIndexMaxAge", &maxtempdelay, NULL, 10}
 		,{ "RedirMax", &redirmax, NULL, 10}
+		,{ "VfileUseRangeOps", &vrangeops, NULL, 10}
 
 		,{ "DirPerms", &dirperms, NULL, 8}
 		,{ "FilePerms", &fileperms, NULL, 8}
@@ -208,6 +212,9 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 {
 	tCfgIter itor(szFilename);
 	itor.reader.CheckGoodState(true);
+
+	NoCaseStringMap dupeCheck;
+
 	while(itor.Next())
 	{
 #ifdef DEBUG
@@ -218,7 +225,7 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 		if(stmiss != pos)
 			itor.sLine.erase(pos);
 
-		if(! SetOption(itor.sLine))
+		if(! SetOption(itor.sLine, false, &dupeCheck))
 			BARF("Error reading main options, terminating.");
 	}
 	return true;
@@ -396,7 +403,8 @@ struct tHookHandler: public tRepoData::IHookHandler, public lockable
 		if (0 >= --m_nRefCnt)
 		{
 			//system(cmdRel.c_str());
-			downTimeNext= ::time(0) + downDuration;
+			downTimeNext = ::time(0) + downDuration;
+			g_victor.RunLatestAt(downTimeNext);
 		}
 	}
 	virtual void JobConnect()
@@ -547,7 +555,7 @@ inline void _FetchMimeTypes()
 	}
 }
 
-bool SetOption(const string &sLine, bool bQuiet)
+bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 {
 
 	string key, value;
@@ -560,9 +568,31 @@ bool SetOption(const string &sLine, bool bQuiet)
 	int nNumBase(10);
 
 	if ( NULL != (psTarget = GetStringPtr(key)))
+	{
+
+		if(pDupeCheck && !bQuiet)
+		{
+			mstring &w = (*pDupeCheck)[key];
+			if(w.empty())
+				w = value;
+			else
+				cerr << "WARNING: " << key << " was previously set to " << w << endl;
+		}
+
 		*psTarget=value;
+	}
 	else if ( NULL != (pnTarget = GetIntPtr(key, nNumBase)))
 	{
+
+		if(pDupeCheck && !bQuiet)
+		{
+			mstring &w = (*pDupeCheck)[key];
+			if(w.empty())
+				w = value;
+			else
+				cerr << "WARNING: " << key << " was already set to " << w << endl;
+		}
+
 		const char *pStart=value.c_str();
 		if(! *pStart)
 		{
@@ -791,7 +821,7 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 	
 	while(itor.Next())
 	{
-		if(debug)
+		if(debug & LOG_DEBUG)
 			cerr << "backends, got line: " << itor.sLine <<endl;
 		
 		// either a URL line or the termination line and hostname/path have been set before
@@ -1191,7 +1221,7 @@ void PostProcConfig()
 
 time_t ExecutePostponed()
 {
-	time_t ret(cleaner::never), now(time(0));
+	time_t ret(g_victor.never()), now(time(0));
 	for (tRepMap::iterator it = repoparms.begin(); it != repoparms.end(); ++it)
 	{
 		if (!it->second.m_pHooks)
@@ -1224,45 +1254,51 @@ time_t ExecutePostponed()
 
 namespace rechecks
 {
-	const UINT rexCount = 3;
-	regex_t rex[rexCount];
+	MYSTD::pair<regex_t, bool> rex[ematchtype_max];
 
 bool CompileExpressions()
 {
-	cmstring *psRex[rexCount] = {&acfg::vfilepat, &acfg::pfilepat, &acfg::wfilepat};
-	for(UINT i=0; i<rexCount; i++)
+	const char *pszExpStrings[] = {
+			acfg::pfilepat.c_str(), acfg::vfilepat.c_str(), acfg::wfilepat.c_str(),
+			BADSTUFF_PATTERN, PTHOSTS_PATTERN.c_str()
+	};
+	for(UINT i=0; i<_countof(pszExpStrings); i++)
 	{
-		int nErr=regcomp(&rex[i], psRex[i]->c_str(), REG_EXTENDED);
+		if(! pszExpStrings[i] || ! pszExpStrings[i][0] )
+			continue;
+
+		int nErr=regcomp(&rex[i].first, pszExpStrings[i], REG_EXTENDED);
 		if(nErr)
 		{
 			char buf[1024];
-			regerror(nErr,  &rex[i], buf, sizeof(buf));
+			regerror(nErr,  &rex[i].first, buf, sizeof(buf));
 			buf[_countof(buf)-1]=0; // better be safe...
-			MYSTD::cerr << buf << ": " << *(psRex[i]) << MYSTD::endl;
+			MYSTD::cerr << buf << ": " << pszExpStrings[i] << MYSTD::endl;
 			return false;
 		}
+		rex[i].second=true;
 	}
 	return true;
 }
 
-UINT GetFiletype(const string & in) {
-	//LOGSTART("rechecks::getFiletype");
-    //dbgline;
-    if(!regexec(&rex[0], in.c_str(), 0, NULL, 0))
-        return FILE_INDEX;
-    //dbgline;
-    if(!regexec(&rex[1], in.c_str(), 0, NULL, 0))
-        return FILE_PKG;
-    //dbgline;
-    return FILE_INVALID;
+bool Match(cmstring &in, eMatchType type)
+{
+//	LOGSTARTs("MatchPattern");
+//	LOG("type: " << type << in);
+	if(!rex[type].second)
+		return false;
+
+	bool bRes = !regexec(&rex[type].first, in.c_str(), 0, NULL, 0);
+//	LOG("result: " << (bRes ? "true" : "false"));
+	return bRes;
 }
 
-bool MatchWhitelist(const string & in)
-{
-	LOGSTART2s("MatchWhiteList", in);
-	bool bPersistent=!regexec(&rex[2], in.c_str(), 0, NULL, 0);
-	LOG("is " << (bPersistent ? "persistent" : "mortal") <<  " file");
-	return bPersistent;
+eMatchType GetFiletype(const string & in) {
+	if (Match(in, FILE_INDEX))
+		return FILE_INDEX;
+	if (Match(in, FILE_PKG))
+		return FILE_PKG;
+	return FILE_INVALID;
 }
 
 deque<regex_t> vecReqPatters, vecTgtPatterns;
@@ -1313,8 +1349,6 @@ bool CompileUncExpressions(const string & req, const string & resolved)
 	return true;
 }
 
-
-
 bool MatchUncacheableRequest(const string & in)
 {
 	LOGSTART2s("MatchUncacheableRequest", in << " against " << vecReqPatters.size() << " patterns");
