@@ -11,17 +11,10 @@ using namespace MYSTD;
 namespace acfg
 {
 
-#if defined(__CYGWIN32__) || defined(__CYGWIN__)
-#warning Cygwin seems to have an ancient regex engine which does not distuinguish between greedy and non-greedy repetition, and defaults to non-greedy
-#define INFOLDER "(^|.*/)"
-#else
-#define INFOLDER "(^|.*?/)"
-#endif
-
 string cachedir("/var/tmp"), logdir("/var/tmp"), fifopath, pidfile, reportpage,
 confdir, adminauth, bindaddr, mirrorsrcs, suppdir("/usr/lib/apt-cacher-ng");
 
-
+#define INFOLDER "(^|.*/)"
 #define COMPRLIST "(\\.gz|\\.bz2|\\.lzma|\\.xz)"
 #define ALXPATTERN ".*\\.(db|files|abs)(\\.tar"COMPRLIST")?"
 #define COMPOPT COMPRLIST"?"
@@ -31,15 +24,15 @@ confdir, adminauth, bindaddr, mirrorsrcs, suppdir("/usr/lib/apt-cacher-ng");
 string pfilepat(".*(\\.d?deb|\\.rpm|\\.dsc|\\.tar"COMPRLIST"(\\.gpg)?"
 		"|\\.diff"COMPRLIST"|\\.jigdo|\\.template|changelog|copyright"
 		"|\\.udeb|\\.debdelta|\\.diff/.*\\.gz|(Devel)?ReleaseAnnouncement(\\?.*)?"
-    "|[a-f0-9]+-(susedata|updateinfo|primary|deltainfo).xml.gz" //opensuse, index data, hash in filename
+		"|[a-f0-9]+-(susedata|updateinfo|primary|deltainfo).xml.gz" //opensuse, index data, hash in filename
 		"|fonts/(final/)?[a-z]+32.exe(\\?download.*)?" // msttcorefonts, fonts/final/comic32.exe /corefonts/comic32.exe plus SF's parameters
 		"|/dists/.*/installer-[^/]+/[0-9][^/]+/images/.*" // d-i stuff with revision
 ")$");
 
-string vfilepat( INFOLDER
+string vfilepat(INFOLDER
 		"(Index|Packages"COMPOPT"|InRelease|Release|Release\\.gpg|"
 		"Sources"COMPOPT"|release|index\\.db-.*\\.gz|Contents-[^/]*"COMPOPT
-		"|pkglist[^/]*\\.bz2|rclist[^/]*\\.bz2|/meta-release[^/]*|Translation[^/]*"COMPOPT
+		"|pkglist[^/]*\\.bz2|rclist[^/]*\\.bz2|meta-release[^/]*|Translation[^/]*"COMPOPT
 		"|MD5SUMS|SHA1SUMS" // d-i stuff
 		"|((setup|setup-legacy)(\\.ini|\\.bz2|\\.hint)(\\.sig)?)|mirrors\\.lst" // cygwin
 		"|repo(index|md)\\.xml(\\.asc|\\.key)?|directory\\.yast" // opensuse
@@ -47,8 +40,8 @@ string vfilepat( INFOLDER
 		"|filelists\\.xml\\.gz|filelists\\.sqlite\\.bz2|repomd\\.xml" // SL, http://ra.khe.sh/computers/linux/apt-cacher-ng-with-yum.html
 		"|packages\\.[a-zA-Z][a-zA-Z]\\.gz|info\\.txt|license\\.tar\\.gz|license\\.zip" //opensuse
 		"|"ALXPATTERN // Arch Linux
-		"|metalink\\?repo|.*prestodelta\\.xml\\.gz" // fedora core, maybe... never worked reliably
-		")$" // end of filename paterns. Any other some specific URLs?
+		"|metalink\\?repo|.*prestodelta\\.xml\\.gz|repodata/.*\\.(xml|sqlite)"COMPRLIST // CentOS
+		")$" // end of only-filename paterns
 		"|/dists/.*/installer-[^/]+/[^0-9][^/]+/images/.*"); // d-i stuff but not containing a number (year) in the revision directory (like "current", "beta", ...)
 
 //string wfilepat( VPATPREFIX  "(Release|Release\\.gpg|release|meta-release|Translation[^/]*\\.bz2)$");
@@ -67,7 +60,7 @@ string wfilepat(INFOLDER
 
 int offlinemode(false), verboselog(true), stupidfs(false), forcemanaged(false),
 extreshhold(20), tpstandbymax(8), tpthreadmax(-1), dirperms(00755), fileperms(00664),
-keepnver(0), maxtempdelay(27);
+keepnver(0), maxtempdelay(27), vrangeops(1);
 
 int dnscachetime(1800), dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(60), updinterval(0),
