@@ -8,6 +8,7 @@
 #include "acfg.h"
 #include "fileio.h"
 #include "conserver.h"
+#include "cleaner.h"
 
 #include <iostream>
 using namespace MYSTD;
@@ -18,6 +19,8 @@ using namespace MYSTD;
 #include <signal.h>
 #include <errno.h>
 
+//#include "job.h"
+
 static void usage();
 static void _TestCacheDir();
 void term_handler(int signum);
@@ -56,6 +59,11 @@ int main(int argc, char **argv)
 
 // PLAYGROUND
 
+   /*
+   cerr << sizeof(job) << endl;
+   exit(1);
+*/
+
 /*
  * Let's be another csum tool...
 
@@ -90,8 +98,13 @@ int main(int argc, char **argv)
 	/*
 	bool Bz2compressFile(const char *, const char*);
 	return ! Bz2compressFile(argv[1], argv[2]);
-	*/
 
+
+	char tbuf[40];
+	FormatCurrentTime(tbuf);
+	MYSTD::cerr << tbuf << MYSTD::endl;
+	exit(1);
+*/
 	const char *toBase64=getenv("TOBASE64");
 	if(toBase64)
 	{
@@ -172,7 +185,8 @@ int main(int argc, char **argv)
 	acfg::PostProcConfig();
     
     _TestCacheDir();
-
+    //sleep(20);
+    //g_victor.Start();
 	conserver::Setup();
 	
 	if (bForceCleanup)
@@ -254,7 +268,8 @@ void term_handler(int signum)
 	case (SIGTERM):
 	case (SIGINT):
 	case (SIGQUIT):
-		aclog::close(false);
+	g_victor.Stop();
+	aclog::close(false);
 	// and then terminate...
 #if 0
 		{
