@@ -72,7 +72,7 @@ void tCacheMan::AddIFileCandidate(const string &sPathRel)
 		return;
 
 	enumIndexType t;
-	if ( (FILE_INDEX == rechecks::GetFiletype(sPathRel)
+	if ( (rechecks::FILE_INDEX == rechecks::GetFiletype(sPathRel)
 	// SUSE stuff, not volatile but also contains file index data
 	|| endsWithSzAr(sPathRel, ".xml.gz") )
 	&& (t=GuessIndexTypeFromURL(sPathRel)) != EIDX_UNSUPPORTED)
@@ -293,11 +293,15 @@ bool tCacheMan::Download(const MYSTD::string & sFilePathRel, bool bIndexFile,
 		else
 		{
 			FiStatus initState = fi->Setup(bIndexFile);
-			if(FIST_COMPLETE == initState && !bIndexFile)
+			if(FIST_COMPLETE == initState)
 			{
 				SendFmt()<<"Checking "<<sFilePathRel<<"... (complete)<br>\n";
 				return true;
 			}
+			if(initState > FIST_COMPLETE)
+			{
+				GOTOREPMSG(fi->GetHeader().frontLine);
+			}
 			if(nVerbosity)
 				SendFmt()<< (bIndexFile?"Checking/Updating " : "Downloading ")
 				<<sFilePathRel<<"...\n";
@@ -647,7 +651,7 @@ class fakeCon : public ::tCacheMan::tGetItemDelegate
 	cmstring & sTempDataRel, &sReferencePathRel;
 	off_t nGotSize;
 	fakeCon(cmstring & a, cmstring &b) :
-			sTempDataRel(a), sReferencePathRel(b)
+			sTempDataRel(a), sReferencePathRel(b), nGotSize(-1)
 	{
 	}
 	operator tFileItemPtr()
@@ -723,13 +727,15 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 		{
 		}
 		// noone else should attempt to store file through it
-		virtual bool DownloadStartedStoreHeader(const header &, const char *)
+		virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &, const char *,
+				bool)
 		{
 			return false;
 		}
 		virtual bool Inject(cmstring &fromRel, const header &head)
 		{
 			m_head = head;
+			mkbasedir(m_sPathAbs);
 			if(head.StoreToFile(m_sPathAbs+".head")<=0)
 				return false;
 
@@ -760,7 +766,7 @@ bool tCacheMan::Inject(cmstring &from, cmstring &to,
 
 			if (Setup(true) >= FIST_LOCALERROR)
 				return false;
-			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL))
+			if (!fileitem_with_storage::DownloadStartedStoreHeader(head, NULL, false))
 				return false;
 			if(!StoreFileData(data.GetBuffer(), data.GetSize()) || ! StoreFileData(NULL, 0))
 				return false;
@@ -1862,6 +1868,10 @@ void tCacheMan::_ProcessSeenIndexFiles()
 
 void tCacheMan::AddDelCbox(cmstring &sFileRel)
 {
+	MYSTD::pair<tStrSet::iterator,bool> ck = m_delCboxFilter.insert(sFileRel);
+	if(! ck.second)
+		return;
+
 	m_bShowControls=true;
 	SendFmt() << "<label><input type=\"checkbox\" name=\"kf" << (nKillLfd++)
 			<< "\" value=\"" << sFileRel << "\">Tag</label>";
