@@ -1,6 +1,7 @@
 
 #include "meta.h"
 #include "caddrinfo.h"
+#include "sockio.h"
 #include "acfg.h"
 #include "debug.h"
 #include "lockable.h"
@@ -97,6 +98,9 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 	if (p->Resolve(sHostname, sPort, sErrorMsgBuf))
 	{
 		p->m_nExpTime = time(0) + acfg::dnscachetime;
+#ifndef MINIBUILD
+		g_victor.RunLatestAt(p->m_nExpTime);
+#endif
 		p->unlock();
 	}
 	else // not good, remove from the cache again
@@ -113,7 +117,7 @@ CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string
 time_t CAddrInfo::ExpireCache()
 {
 	lockguard g(&lockDnsCache);
-	time_t now(time(0)), ret(cleaner::never);
+	time_t now(time(0)), ret(g_victor.never());
 
 	for(map<string, CAddrInfo::SPtr>::iterator it=mapDnsCache.begin();
 			it!=mapDnsCache.end(); )
