@@ -23,6 +23,7 @@ using namespace MYSTD;
 con::con(int fdId, const char *c) :
 	m_confd(fdId),
     m_bStopActivity(false),
+    m_dlerthr(0),
     m_pDlClient(NULL),
     m_pTmpHead(NULL)
 {
@@ -31,9 +32,6 @@ con::con(int fdId, const char *c) :
 
     LOGSTART2("con::con", "fd: " << fdId << ", clienthost: " << c);
 
-    // actually, just make sure that it runs right before, even if the cleaner thread is busy
-    fileitem::DoDelayedUnregAndCheck();
-
 #ifdef DEBUG
     m_nProcessedJobs=0;
 #endif
@@ -61,44 +59,62 @@ con::~con() {
     	delete m_pDlClient;
     	m_pDlClient=NULL;
     }
-    
     if(m_pTmpHead)
     {
     	delete m_pTmpHead;
     	m_pTmpHead=NULL;
     }
-    				
     aclog::flush();
-
-    // some extra work might have been postponed
-    g_victor.Do();
 }
 
-class SoapPassThrough
+class RawPassThrough
 {
 	protected:
 	tTcpHandlePtr m_spOutCon;
 
-public:
-	bool TrySoapPassThrough(acbuf &clientBufIn, int fdClient)
-	{
-
 #define POSTMARK "POST http://bugs.debian.org:80/"
 #define FAKEMARK "POST                          /"
 
-		if(clientBufIn.size()<_countof(POSTMARK)-1)
-			return false;
-		if(strncmp(clientBufIn.rptr(), POSTMARK, _countof(POSTMARK)-1))
-			return false;
 
+public:
+	inline bool CheckListbugs(const header &ph)
+	{
+		return (0 == ph.frontLine.compare(0, _countof(POSTMARK) - 1, POSTMARK));
+	}
+	void PassThrough(acbuf &clientBufIn, int fdClient,
+			bool listBugsMode, cmstring *pConTgt)
+	{
 		string sErr;
-		m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", "80", sErr);
+		tSS clientBufOut;
+		clientBufOut.init(32*1024); // out to be enough for any BTS response
+
+		if(listBugsMode)
+			m_spOutCon = tcpconnect::CreateConnected("bugs.debian.org", "80", sErr);
+		else if(pConTgt)
+		{
+			tHttpUrl url;
+			if(!url.SetHttpUrl(*pConTgt))
+				return;
+			m_spOutCon = tcpconnect::CreateConnected(url.sHost , url.sPort, sErr);
+
+			if(m_spOutCon)
+				clientBufOut << "HTTP/1.0 200 Connection established\r\n\r\n";
+			else
+			{
+				clientBufOut << "HTTP/1.0 502 CONNECT error: " << sErr << "\r\n\r\n";
+				while(!clientBufOut.empty())
+					clientBufOut.syswrite(fdClient);
+			}
+
+		}
+		else
+		{
+			ASSERT(!"huh?");
+			return;
+		}
 
 		if(!m_spOutCon)
-			return false;
-
-		acbuf clientBufOut; // use custom buffers?
-		clientBufOut.init(32*1024); // out to be enough for any BTS response
+			return;
 
 		// for convenience
 		int ofd = m_spOutCon->GetFD();
@@ -128,7 +144,7 @@ public:
 
 			int nReady=select(maxfd, &rfds, &wfds, NULL, NULL);
 			if (nReady<0)
-				return false;
+				return;
 
 			if(FD_ISSET(ofd, &wfds))
 			{
@@ -145,28 +161,28 @@ public:
 				}
 
 				if(serverBufOut.syswrite(ofd)<0)
-					return false;
+					return;
 			}
 
 			if(FD_ISSET(fdClient, &wfds))
 			{
 				if(clientBufOut.syswrite(fdClient)<0)
-					return false;
+					return;
 			}
 
 			if(FD_ISSET(ofd, &rfds))
 			{
 				if(serverBufIn.sysread(ofd)<=0)
-					return true;
+					return;
 			}
 
 			if(FD_ISSET(fdClient, &rfds))
 			{
 				if(clientBufIn.sysread(fdClient)<=0)
-					return true;
+					return;
 			}
 		}
-		return false;
+		return;
 	};
 };
 
@@ -242,28 +258,60 @@ void con::WorkLoop() {
 				if(!m_pTmpHead)
 					return; // no resources? whatever
 				
-				int nConsumed=m_pTmpHead->LoadFromBuf(inBuf.rptr(), inBuf.size());
-				ldbg("header parsed how? " << nConsumed);
-				if(nConsumed==0)
+				int nHeadBytes=m_pTmpHead->LoadFromBuf(inBuf.rptr(), inBuf.size());
+				ldbg("header parsed how? " << nHeadBytes);
+				if(nHeadBytes == 0)
 				{ // Either not enough data received, or buffer full; make space and retry
 					inBuf.move();
 					break;
 				}
-				if(nConsumed<0)
+				if(nHeadBytes < 0)
 				{
-					 // also must be identified before
-					if( acfg::forwardsoap && ! m_sClientHost.empty() )
-					{
-						SoapPassThrough pt;
-						if(pt.TrySoapPassThrough(inBuf, m_confd))
-							return;
-					}
-
 					ldbg("Bad request: " << inBuf.rptr() );
 					return;
 				}
-				if (nConsumed>0)
-					inBuf.drop(nConsumed);
+
+				// also must be identified before
+				if (m_pTmpHead->type == header::POST)
+				{
+					if (acfg::forwardsoap && !m_sClientHost.empty())
+					{
+						RawPassThrough pt;
+						if (pt.CheckListbugs(*m_pTmpHead))
+							pt.PassThrough(inBuf, m_confd, true, NULL);
+						else
+						{
+							ldbg("not bugs.d.o: " << inBuf.rptr());
+						}
+						return;
+					}
+					ldbg("not allowed POST request: " << inBuf.rptr());
+					return;
+				}
+
+				inBuf.drop(nHeadBytes);
+
+				if(m_pTmpHead->type == header::CONNECT)
+				{
+					tSplitWalk iter(&m_pTmpHead->frontLine);
+					if(iter.Next() && iter.Next())
+					{
+						cmstring tgt(iter);
+						if(rechecks::Match(tgt, rechecks::PASSTHROUGH))
+						{
+							RawPassThrough pt;
+							pt.PassThrough(inBuf, m_confd, false, &tgt);
+						}
+						else
+						{
+							tSS response;
+							response << "HTTP/1.0 403 CONNECT denied\r\n\r\n";
+							while(!response.empty())
+								response.syswrite(m_confd);
+						}
+					}
+					return;
+				}
 				
 				if (m_sClientHost.empty()) // may come from wrapper... MUST identify itself
 				{
