@@ -11,6 +11,9 @@
 
 using namespace MYSTD;
 
+// evil hack to simulate random disconnects
+//#define DISCO_FAILURE
+
 static const cmstring sGenericError("567 Unknown download error occured");
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
@@ -36,15 +39,15 @@ struct tDlJob
 		return (!m_pStorage || m_pStorage->GetStatus() >= FIST_LOCALERROR);
 	}
 
-#define EFLAG_MORE 0
-#define EFLAG_DONE 1
-#define EFLAG_DISCON 2
+#define HINT_MORE 0
+#define HINT_DONE 1
+#define HINT_DISCON 2
 #define EFLAG_JOB_BROKEN 4
 #define EFLAG_MIRROR_BROKEN 8
 #define EFLAG_STORE_COLLISION 16
-#define EFLAG_HINT_SWITCH 32
+#define HINT_SWITCH 32
 #define EFLAG_LOST_CON 64
-#define EFLAG_KILL_LAST_FILE 128
+#define HINT_KILL_LAST_FILE 128
 
 	inline const mstring GetPeerName()
 	{
@@ -64,7 +67,7 @@ struct tDlJob
 
 	typedef enum
 	{
-		STATE_GETHEADER, STATE_PROCESS_DATA,
+		STATE_GETHEADER, STATE_REGETHEADER, STATE_PROCESS_DATA,
 		STATE_GETCHUNKHEAD, STATE_PROCESS_CHUNKDATA, STATE_GET_CHUNKTRAILER,
 		STATE_FINISHJOB
 	} tDlState;
@@ -86,7 +89,6 @@ struct tDlJob
 		m_eReconnectASAP = 0;
 		m_DlState = STATE_GETHEADER;
 		m_pCurBackend = NULL;
-
 		if (m_pStorage)
 			m_pStorage->IncDlRefCount();
 	}
@@ -158,16 +160,21 @@ struct tDlJob
 
 			lockguard g(m_pStorage.get());
 			const header &pHead = m_pStorage->GetHeaderUnlocked();
-			const char *p = pHead.h[header::LAST_MODIFIED];
 
 			if (m_pStorage->m_bCheckFreshness)
 			{
-				if (p)
+				if (pHead.h[header::LAST_MODIFIED])
 				{
-					bSetIfRange = true;
-					bSetRange = true;
+					if (acfg::vrangeops > 0)
+					{
+						bSetIfRange = true;
+						bSetRange = true;
+					}
+					else if(acfg::vrangeops == 0)
+					{
+						head << "If-Modified-Since: " << pHead.h[header::LAST_MODIFIED] << "\r\n";
+					}
 				}
-				// else no date available, cannot rely on anything
 			}
 			else
 			{
@@ -190,7 +197,7 @@ struct tDlJob
 				bSetRange = bSetIfRange = false;
 */
 
-			/* use APT's trick - set the starting position one byte lower -
+			/* use APT's old trick - set the starting position one byte lower -
 			 * this way the server has to send at least one byte if the assumed
 			 * position is correct, and we never get a 416 error (one byte
 			 * waste is acceptable).
@@ -204,7 +211,7 @@ struct tDlJob
 			}
 
 			if (bSetIfRange)
-				head << "If-Range: " << p << "\r\n";
+				head << "If-Range: " << pHead.h[header::LAST_MODIFIED] << "\r\n";
 		}
 
 		if (m_pStorage->m_bCheckFreshness)
@@ -238,7 +245,10 @@ struct tDlJob
 				break;
 
 			if (!m_pStorage->StoreFileData(inBuf.rptr(), nToStore))
-				return EFLAG_DISCON | EFLAG_JOB_BROKEN;
+			{
+				sErrorMsg = "502 Could not store data";
+				return HINT_DISCON | EFLAG_JOB_BROKEN;
+			}
 
 			m_nRest -= nToStore;
 			inBuf.drop(nToStore);
@@ -251,9 +261,9 @@ struct tDlJob
 			m_DlState = (STATE_PROCESS_DATA == m_DlState) ? STATE_FINISHJOB : STATE_GETCHUNKHEAD;
 		}
 		else
-			return EFLAG_MORE; // will come back
+			return HINT_MORE; // will come back
 
-		return EFLAG_HINT_SWITCH;
+		return HINT_SWITCH;
 	}
 
 	/*!
@@ -264,28 +274,34 @@ struct tDlJob
 	{
 		LOGSTART("tDlJob::ProcessIncomming");
 		if (!m_pStorage)
-			return EFLAG_DISCON | EFLAG_JOB_BROKEN;
+		{
+			sErrorMsg = "502 Bad cache descriptor";
+			return HINT_DISCON | EFLAG_JOB_BROKEN;
+		}
 
 		for (;;) // returned by explicit error (or get-more) return
 		{
 			ldbg("switch: " << m_DlState);
 
-			if ( STATE_GETHEADER == m_DlState)
+			if (STATE_GETHEADER == m_DlState ||  STATE_REGETHEADER == m_DlState)
 			{
 				ldbg("STATE_GETHEADER");
 				header h;
 				if (inBuf.size() == 0)
-					return EFLAG_MORE;
+					return HINT_MORE;
+
+				bool bHotItem = (m_DlState == STATE_REGETHEADER);
 				dbgline;
+
 				int l = h.LoadFromBuf(inBuf.rptr(), inBuf.size());
 				if (0 == l)
-					return EFLAG_MORE;
+					return HINT_MORE;
 				if (l<0)
 				{
 					dbgline;
 					sErrorMsg = "500 Invalid header";
 					// can be followed by any junk... drop that mirror, previous file could also contain bad data
-					return EFLAG_MIRROR_BROKEN | EFLAG_DISCON | EFLAG_KILL_LAST_FILE;
+					return EFLAG_MIRROR_BROKEN | HINT_DISCON | HINT_KILL_LAST_FILE;
 				}
 
 				ldbg("contents: " << MYSTD::string(inBuf.rptr(), l));
@@ -294,7 +310,7 @@ struct tDlJob
 				{
 					sErrorMsg = "500 Unexpected response type";
 					// smells fatal...
-					return EFLAG_MIRROR_BROKEN | EFLAG_DISCON;
+					return EFLAG_MIRROR_BROKEN | HINT_DISCON;
 				}
 				ldbg("GOT, parsed: " << h.frontLine);
 
@@ -307,7 +323,7 @@ struct tDlJob
 						if (endsWith(m_fileUri.sPath, *it))
 						{
 							sErrorMsg = "500 Bad mirror";
-							return EFLAG_DISCON | EFLAG_MIRROR_BROKEN;
+							return HINT_DISCON | EFLAG_MIRROR_BROKEN;
 						}
 					}
 				}
@@ -318,11 +334,19 @@ struct tDlJob
 				if (pCon && 0 == strcasecmp(pCon, "close"))
 				{
 					ldbg("Peer wants to close connection after request");
-					m_eReconnectASAP = EFLAG_DISCON;
+					m_eReconnectASAP = HINT_DISCON;
 				}
 
 				if (m_pStorage->m_bHeadOnly)
+				{
 					m_DlState = STATE_FINISHJOB;
+				}
+				// the only case where we expect a 304
+				else if(h.getStatus() == 304 && acfg::vrangeops == 0)
+				{
+					m_pStorage->SetupComplete();
+					m_DlState = STATE_FINISHJOB;
+				}
 				else if (h.h[header::TRANSFER_ENCODING]
 						&& 0 == strcasecmp(h.h[header::TRANSFER_ENCODING], "chunked"))
 				{
@@ -335,18 +359,27 @@ struct tDlJob
 					if (!h.h[header::CONTENT_LENGTH])
 					{
 						sErrorMsg = "500 Missing Content-Length";
-						return EFLAG_DISCON | EFLAG_JOB_BROKEN;
+						return HINT_DISCON | EFLAG_JOB_BROKEN;
 					}
 					// may support such endless stuff in the future but that's too unreliable for now
 					m_nRest = atoofft(h.h[header::CONTENT_LENGTH]);
 					m_DlState = STATE_PROCESS_DATA;
 				}
 				h.set(header::XORIG, RemoteUri());
-				if (!m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr()))
+				fileitem_base::SuccessWithTransErrorFlag headStored = m_pStorage->DownloadStartedStoreHeader(h, inBuf.rptr(), bHotItem );
+				if(!headStored && headStored.xdata)
+					return EFLAG_LOST_CON | HINT_DISCON; // recoverable
+
+				if(!headStored)
 				{
 					ldbg("Item dl'ed by others or in error state --> drop it, reconnect");
 					m_DlState = STATE_PROCESS_DATA;
-					return EFLAG_DISCON | EFLAG_JOB_BROKEN | EFLAG_STORE_COLLISION;
+					sErrorMsg = "502 Cache descriptor busy";
+/*					header xh = m_pStorage->GetHeader();
+					if(xh.frontLine.length() > 12)
+						sErrorMsg = sErrorMsg + " (" + xh.frontLine.substr(12) + ")";
+						*/
+					return HINT_DISCON | EFLAG_JOB_BROKEN | EFLAG_STORE_COLLISION;
 				}
 			}
 			else if (m_DlState == STATE_PROCESS_CHUNKDATA || m_DlState ==  STATE_PROCESS_DATA)
@@ -354,7 +387,7 @@ struct tDlJob
 				// similar states, just handled differently afterwards
 				ldbg("STATE_GETDATA");
 				uint_fast8_t res = NewDataHandler(inBuf);
-				if (EFLAG_HINT_SWITCH != res)
+				if (HINT_SWITCH != res)
 					return res;
 			}
 			else if (m_DlState == STATE_FINISHJOB)
@@ -362,7 +395,7 @@ struct tDlJob
 				ldbg("STATE_FINISHJOB");
 				m_DlState = STATE_GETHEADER;
 				m_pStorage->StoreFileData(NULL, 0);
-				return EFLAG_DONE | m_eReconnectASAP;
+				return HINT_DONE | m_eReconnectASAP;
 			}
 			else if (m_DlState == STATE_GETCHUNKHEAD)
 			{
@@ -379,11 +412,14 @@ struct tDlJob
 				if (!inBuf.size() || NULL == (crlf = strstr(pStart, "\r\n")))
 				{
 					inBuf.move();
-					return EFLAG_MORE;
+					return HINT_MORE;
 				}
 				UINT len(0);
 				if (1 != sscanf(pStart, "%x", &len))
+				{
+					sErrorMsg = "500 Invalid data stream";
 					return EFLAG_JOB_BROKEN; // hm...?
+				}
 				inBuf.drop(crlf + 2 - pStart);
 				if (len > 0)
 				{
@@ -396,11 +432,11 @@ struct tDlJob
 			else if (m_DlState == STATE_GET_CHUNKTRAILER)
 			{
 				if (inBuf.size() < 2)
-					return EFLAG_MORE;
+					return HINT_MORE;
 				const char *pStart(inBuf.c_str());
 				const char *crlf(strstr(pStart, "\r\n"));
 				if (!crlf)
-					return EFLAG_MORE;
+					return HINT_MORE;
 
 				if (crlf == pStart) // valid empty line -> done here
 				{
@@ -412,6 +448,7 @@ struct tDlJob
 			}
 		}
 		ASSERT(!"unreachable");
+		sErrorMsg = "502 Invalid state";
 		return EFLAG_JOB_BROKEN;
 	}
 
@@ -585,14 +622,14 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			LOG(fer);
 #endif
 			sErrorMsg = string("500 Internal malfunction, ") + fer;
-			return EFLAG_DISCON|EFLAG_JOB_BROKEN|EFLAG_MIRROR_BROKEN;
+			return HINT_DISCON|EFLAG_JOB_BROKEN|EFLAG_MIRROR_BROKEN;
 		}
 		else if (r == 0) // looks like a timeout
 		{
 			dbgline;
 			sErrorMsg = "500 Connection timeout";
 			// was there anything to do at all?
-			return qActive.empty() ? EFLAG_HINT_SWITCH : (EFLAG_DISCON|EFLAG_JOB_BROKEN);
+			return qActive.empty() ? HINT_SWITCH : (HINT_DISCON|EFLAG_JOB_BROKEN);
 		}
 
 		if (FD_ISSET(m_wakepipe[0], &rfds))
@@ -600,7 +637,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			dbgline;
 			for (int tmp; read(m_wakepipe[0], &tmp, 1) > 0;)
 				;
-			return EFLAG_HINT_SWITCH;
+			return HINT_SWITCH;
 		}
 
 		if (fd>=0 && FD_ISSET(fd, &wfds))
@@ -612,12 +649,12 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			ldbg("Sent " << s << " bytes from " << m_sendBuf.length());
 			if (s < 0)
 			{
+				// EAGAIN is weird but let's retry later, otherwise reconnect
 				if (errno != EAGAIN && errno != EINTR)
 				{
-					sErrorMsg = "502 Peer communication error";
-					return EFLAG_JOB_BROKEN|EFLAG_DISCON;
+					sErrorMsg = "502 Send failed";
+					return EFLAG_LOST_CON;
 				}
-				// else retry later...
 			}
 			else
 				m_sendBuf.drop(s);
@@ -627,10 +664,36 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 		{
 
 			r = m_inBuf.sysread(fd);
-			if (r <= 0) // there must be data. If no -> disconnected.
+
+
+#ifdef DISCO_FAILURE
+#warning hej
+			static int fakeFail=3;
+			if( fakeFail-- < 0)
+			{
+//				LOGLVL(LOG_DEBUG, "\n#################\nFAKING A FAILURE\n###########\n");
+				r=0;
+				fakeFail=10;
+				errno = EROFS;
+				r = -errno;
+			}
+#endif
+
+			if(r == -EAGAIN || r == -EWOULDBLOCK)
+			{
+				ldbg("why EAGAIN/EINTR after getting it from select?");
+//				timespec sleeptime = { 0, 432000000 };
+//				nanosleep(&sleeptime, NULL);
+				goto loop_again;
+			}
+			else if (r == 0)
+			{
+				sErrorMsg = "502 Connection closed";
+				return EFLAG_LOST_CON;
+			}
+			else if (r < 0) // other error, might reconnect
 			{
 				dbgline;
-
 #ifdef MINIBUILD
 				sErrorMsg = "502 EPIC FAIL";
 #else
@@ -658,17 +721,17 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 						"... incoming data processing result: " << res
 						<< ", emsg: " << qActive.front()->sErrorMsg);
 
-				if (EFLAG_MORE == res)
+				if (HINT_MORE == res)
 					goto loop_again;
 
-				if (EFLAG_DONE & res)
+				if (HINT_DONE & res)
 				{
 					// just in case that server damaged the last response body
 					con->NameLastFile(qActive.front()->m_pStorage->m_sPathRel);
 
 					qActive.pop_front();
-					if (EFLAG_DISCON & res)
-						return EFLAG_DISCON; // with cleaned flags
+					if (HINT_DISCON & res)
+						return HINT_DISCON; // with cleaned flags
 
 					LOG(
 							"job finished. Has more? " << qActive.size()
@@ -677,7 +740,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 					if (qActive.empty())
 					{
 						LOG("Need more work");
-						return EFLAG_HINT_SWITCH;
+						return HINT_SWITCH;
 					}
 
 					LOG("Extract more responses");
@@ -686,20 +749,20 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 				else // error handling, pass to main loop
 				{
 
-					if(EFLAG_KILL_LAST_FILE & res)
+					if(HINT_KILL_LAST_FILE & res)
 						con->KillLastFile();
 
 					setIfNotEmpty(sErrorMsg, qActive.front()->sErrorMsg);
 					return res;
 				}
 			}
-			return EFLAG_DONE; // input buffer consumed
+			return HINT_DONE; // input buffer consumed
 		}
 	}
 
 	ASSERT(!"Unreachable");
 	sErrorMsg = "500 Internal failure";
-	return EFLAG_JOB_BROKEN|EFLAG_DISCON;
+	return EFLAG_JOB_BROKEN|HINT_DISCON;
 }
 
 inline void CleanRunning(tDljQueue &inpipe)
@@ -742,8 +805,8 @@ void dlcon::WorkLoop()
 	bool bStopRequesting=false; // hint to stop adding request headers until the connection is restarted
 
 	int nLostConTolerance=0;
-#define MAX_RETRY_USED 4
-#define MAX_RETRY_FRESH 2
+#define MAX_RETRY_USED 8
+#define MAX_RETRY_FRESH 5
 
 	while(true) // outer loop: jobs, connection handling
 	{
@@ -831,7 +894,11 @@ void dlcon::WorkLoop()
         				<< ", good config: " << bGoodConfig);
 
         		if(!bGoodConfig)
+        		{
+        			// something weird happened to it, drop it and let the client care
+        			m_qNewjobs.pop_front();
         			continue;
+        		}
 
         		if(acfg::proxy_info.sHost.empty()
         				// needs to send them for the connected target host
@@ -873,7 +940,7 @@ void dlcon::WorkLoop()
         	bStopRequesting=false;
         }
 
-        if(0==(loopRes&(EFLAG_DISCON|EFLAG_JOB_BROKEN|EFLAG_LOST_CON|EFLAG_MIRROR_BROKEN))
+        if(0==(loopRes&(HINT_DISCON|EFLAG_JOB_BROKEN|EFLAG_LOST_CON|EFLAG_MIRROR_BROKEN))
         		&& con && inpipe.empty()) // no error bits set, not busy -> recycle it when idle
         {
         	dbgline;
@@ -881,31 +948,45 @@ void dlcon::WorkLoop()
 			continue;
 		}
 
-        if( (EFLAG_DISCON|EFLAG_LOST_CON) & loopRes)
+        if( (HINT_DISCON|EFLAG_LOST_CON) & loopRes)
         {
         	dbgline;
         	con.reset();
         }
 
-        if(EFLAG_LOST_CON & loopRes)
-        {
-        	// disconnected by OS... give it a chance, or maybe not...
-           	if(--nLostConTolerance <= 0 && !inpipe.empty())
-           		BlacklistMirror(inpipe.front(), sErrorMsg);
-           	dbgline;
-        }
+		if ((EFLAG_LOST_CON & loopRes) && !inpipe.empty())
+		{
+			// disconnected by OS... give it a chance, or maybe not...
+			if (--nLostConTolerance <= 0)
+				BlacklistMirror(inpipe.front(), sErrorMsg);
 
-        if(loopRes<=EFLAG_DONE)
+			timespec sleeptime = { 0, 325000000 };
+			nanosleep(&sleeptime, NULL);
+
+			// trying to resume that job secretly, unless user disabled the use of range (we
+			// cannot resync the sending position ATM, throwing errors to user for now)
+			if (acfg::vrangeops <= 0 && inpipe.front()->m_pStorage->m_bCheckFreshness)
+				loopRes |= EFLAG_JOB_BROKEN;
+			else
+				inpipe.front()->m_DlState = tDlJob::STATE_REGETHEADER;
+		}
+
+        if(loopRes & (HINT_DONE|HINT_MORE))
         {
         	nLostConTolerance=MAX_RETRY_FRESH;
+        	sErrorMsg.clear();
         	continue;
         }
 
-        // regular required post-processing done
-        if(EFLAG_HINT_SWITCH == loopRes)
+        //
+        // regular required post-processing done here, now handle special conditions
+        //
+
+
+        if(HINT_SWITCH == loopRes)
         	continue;
 
-        // ok, now needs to resolve the fatal error situation, push the pipelined back to new, etc.
+        // resolving the "fatal error" situation, push the pipelined job back to new, etc.
 
         if( (EFLAG_MIRROR_BROKEN & loopRes) && !inpipe.empty())
         	BlacklistMirror(inpipe.front(), sErrorMsg);
@@ -916,21 +997,21 @@ void dlcon::WorkLoop()
 
         	inpipe.pop_front();
 
-        	// stupid situation, both users downloading the same stuff - and most likely in the same order
-        	// if one downloader runs a step ahead (or usually many steps), drop all items
-        	// already processed by it and try to continue somewhere else.
-        	// This way, the overall number of collisions and reconnects is minimized
         	if(EFLAG_STORE_COLLISION & loopRes)
         	{
-        		// seriously, I want lambdas
-        		CleanRunning(inpipe);
+				// stupid situation, both users downloading the same stuff - and most likely in the same order
+				// if one downloader runs a step ahead (or usually many steps), drop all items
+				// already processed by it and try to continue somewhere else.
+				// This way, the overall number of collisions and reconnects is minimized
+
+        		CleanRunning(inpipe); // seriously, I want lambdas
         		setLockGuard;
         		CleanRunning(m_qNewjobs);
         	}
 
         }
 
-
+        // if the job was not dropped, push it back into the task queue
         {
         	setLockGuard;
         	m_qNewjobs.insert(m_qNewjobs.begin(), inpipe.begin(), inpipe.end());
