@@ -123,7 +123,9 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 				{
 					SendFmt() << "<font color=red>WARNING, header file of " << sPathRel
 							<< " reported too small file size (" << len << " vs. " << lenInfo
-							<< "), invalidating</font><br>\n";
+							<< "), invalidating and removing header</font><br>\n";
+					// kill the header ASAP, no matter which extra exception might apply later
+					ignore_value(::unlink((sPathAbs+".head").c_str()));
 					goto keep_in_trash;
 				}
 			}
@@ -132,13 +134,13 @@ void expiration::HandlePkgEntry(const tRemoteFileInfo &entry, bool bUnpackForCsu
 				tIfileAttribs at = GetFlags(sPathRel);
 				if(!at.parseignore && !at.forgiveDlErrors)
 				{
-					SendFmt()<<"<font color=\"orange\">WARNING, header file missing for "
+					SendFmt()<<"<font color=\"orange\">WARNING, header file missing or damaged for "
 						<<sPathRel<<"</font><br>\n";
 				}
 			}
 		}
 
-		if(m_bByChecksum)
+		if(m_bByChecksum && fprHave.size != 0)
 		{
 
 			bool bSkipDataCheck=false;
@@ -342,6 +344,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 	off_t tagSpace(0);
 	for (tS2DAT::iterator it = m_trashCandSet.begin(); it != m_trashCandSet.end(); it++)
 	{
+		using namespace rechecks;
 		const tFileNdir &k = it->first; // shortcut
 		string sPathRel = k.dirRel+k.file;
 
@@ -351,7 +354,7 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
 		}
 		else
 		{
-			if (rechecks::MatchWhitelist(k.file) || rechecks::MatchWhitelist(sPathRel))
+			if (Match(k.file, WHITELIST) || Match(sPathRel, WHITELIST))
 			{
 				LOG("File not to be removed, ignoring");
 				continue;
@@ -402,8 +405,9 @@ inline void expiration::RemoveAndStoreStatus(bool bPurgeNow)
             it != m_trashCandHeadSet.end(); it++)
 	{
 		string sPathRel(it->dirRel + it->file);
-		if (rechecks::MatchWhitelist(sPathRel) || (endsWithSzAr(sPathRel, ".head")
-				&& rechecks::MatchWhitelist(sPathRel.substr(0, sPathRel.size() - 5))))
+		if (rechecks::Match(sPathRel, rechecks::WHITELIST)
+				|| (endsWithSzAr(sPathRel, ".head")
+				&& rechecks::Match(sPathRel.substr(0, sPathRel.size() - 5), rechecks::WHITELIST)))
 		{
 			continue;
 		}
@@ -464,7 +468,8 @@ void expiration::Action(const string & cmd)
 				"(no further confirmation)<br>\n";
 		return;
 	}
-	if(m_mode==purgeDamaged || m_mode==listDamaged)
+
+	if(m_mode==purgeDamaged || m_mode==listDamaged || m_mode==truncDamaged)
 	{
 		filereader f;
 		if(!f.OpenFile(SABSPATH(FNAME_DAMAGED)))
@@ -475,12 +480,22 @@ void expiration::Action(const string & cmd)
 		mstring s;
 		while(f.GetOneLine(s))
 		{
-			SendFmt() << (m_mode==purgeDamaged ? "Removing " : "") << s << "<br>\n";
-			if(m_mode==purgeDamaged)
+			if(s.empty())
+				continue;
+
+			if(m_mode == purgeDamaged)
 			{
+				SendFmt() << "Removing " << s << "<br>\n";
 				::unlink(SZABSPATH(s));
 				::unlink(SZABSPATH(s+".head"));
 			}
+			else if(m_mode == truncDamaged)
+			{
+				SendFmt() << "Truncating " << s << "<br>\n";
+				ignore_value(::truncate(SZABSPATH(s), 0));
+			}
+			else
+				SendFmt() << s << "<br>\n";
 		}
 		return;
 	}
@@ -637,13 +652,14 @@ bool expiration::ProcessRegular(const string & sPathAbs, const struct stat &stin
 }
 
 
-expiration::expiration(int fd, workType type) : tCacheMan(fd), m_mode(type)
+expiration::expiration(int fd, workType type) : tCacheMan(fd), m_mode(type), m_bIncompleteIsDamaged(false)
 {
 	switch(type)
 	{
 	case list: m_sTypeName="Listing unreferenced"; break;
 	case listDamaged: m_sTypeName="Listing damaged files"; break;
 	case purgeDamaged: m_sTypeName="Removing damaged files"; break;
+	case truncDamaged: m_sTypeName="Truncating damaged files to zero size"; break;
 	case expire:
 	default:
 		m_sTypeName="Expiration"; break;
