@@ -30,7 +30,7 @@ string fileitem_base::GetHttpMsg()
 	return m_head.frontLine;
 }
 
-time_t fileitem_base::m_nEarliestExpiration(0);
+time_t fileitem_base::m_nEarliestExpiration(cleaner::never());
 
 fileitem_base::fileitem_base(MYSTD::string sPath) :
 	condition(),
@@ -224,6 +224,13 @@ FiStatus fileitem_base::Setup(bool bCheckFreshness)
 			if(!p)
 				goto error_clean; // suspicious, cannot use it
 			LOG("check freshness, last modified: " << p );
+
+			// that will cause check by if-mo-only later, needs to be sure about the size here
+			if(acfg::vrangeops == 0
+					&& m_nSizeSeen != atoofft(m_head.h[header::CONTENT_LENGTH], -17))
+			{
+				m_nSizeSeen = 0;
+			}
 		}
 		else
 		{
@@ -310,15 +317,40 @@ bool fileitem_base::SetupClean(bool bForce)
 	return true;
 }
 
+void fileitem_base::SetupComplete()
+{
+	setLockGuard;
+	notifyAll();
+	m_nSizeChecked = m_nSizeSeen;
+	m_status = FIST_COMPLETE;
+}
+
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
 	aclog::errnoFmter f;
-	aclog::err( (sFile+" storage error ["+msg+"]: "+f).c_str());
+	aclog::err( (sFile+" storage error ["+msg+"], last errno: "+f).c_str());
 }
 
 #ifndef MINIBUILD
 
-bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData)
+/*
+#ifdef DEBUG
+#define SET_FRONTLINE(ret, x) tSS __fbuf; __fbuf << "HTTP/1.1 " << x << " @@ " << __FILE__ ":" \
+	<< __LINE__ << " /" << __FUNCTION__; ret=__fbuf;
+#else
+#endif
+*/
+
+#define SET_FRONTLINE(ret, x) ret="HTTP/1.1 "; ret+=x;
+#define SETERROR(x) { m_bAllowStoreData=false; SET_FRONTLINE(m_head.frontLine, x); \
+	m_head.set(header::XORIG, h.h[header::XORIG]); \
+    m_status=FIST_LOCALERROR; _LogWithErrno(x, m_sPathRel); }
+#define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
+#define BOUNCE(x) { SETERROR(x); return false; }
+
+fileitem::SuccessWithTransErrorFlag
+fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const char *pNextData,
+		bool bForcedRestart)
 {
 	LOGSTART("fileitem::DownloadStartedStoreHeader");
 
@@ -326,11 +358,15 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 
 	USRDBG(5, "Download started, storeHeader for " << m_sPathRel << ", current status: " << m_status);
 	
-	if(m_status > FIST_DLPENDING) // already started? error? whatever
+	if(m_status >= FIST_COMPLETE)
+	{
+		USRDBG(0, "Error: item finished and idle, refusing download start");
+		return false;
+	}
+
+	// conflict with another thread's initial attempt? Deny, except for a forced restart
+	if (m_status > FIST_DLPENDING && !bForcedRestart)
 		return false;
-	
-	if(m_status<FIST_DLGOTHEAD)
-		m_status=FIST_DLGOTHEAD; // assume that for now, may become error state...
 	
 	if(m_bCheckFreshness)
 		m_nTimeExpireAt = GetTime() + MAXTEMPDELAY;
@@ -343,26 +379,51 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
 
-#define SETERROR(x) { m_bAllowStoreData=false; m_head.frontLine="HTTP/1.1 "; \
-	m_head.frontLine+=x; m_status=FIST_LOCALERROR; _LogWithErrno(x, m_sPathRel); }
-#define SETERRORKILLFILE(x) { SETERROR(x); goto kill_file; }
-#define BOUNCE(x) { SETERROR(x); return false; }
-
-	m_head.set(header::XORIG, h.h[header::XORIG]);
-
 	string sHeadPath=m_sPathAbs+".head";
-	
-	switch(h.getStatus())
+	int serverStatus = h.getStatus();
+#if 0
+#warning FIXME
+	static UINT fc=1;
+	if(!(++fc % 4))
+	{
+		serverStatus = 416;
+	}
+#endif
+	switch(serverStatus)
 	{
 	case 200:
 	{
-		m_nSizeChecked=0;
-		m_head=h;
+		if(m_status < FIST_DLGOTHEAD)
+			bForcedRestart = false; // behave normally, set all data
+
+		if (bForcedRestart)
+		{
+			if (m_nSizeChecked != 0)
+			{
+				/* shouldn't be here, server should have resumed at the previous position.
+				 * Most likely the remote file was modified after the download started.
+				 */
+				//USRDBG(0, "state: " << m_status << ", m_nsc: " << m_nSizeChecked);
+				BOUNCE("500 Failed to resume remote download");
+			}
+			if(h.h[header::CONTENT_LENGTH] && atoofft(h.h[header::CONTENT_LENGTH])
+					!= atoofft(h.h[header::CONTENT_LENGTH], -2))
+			{
+				BOUNCE("500 Failed to resume remote download, bad length");
+			}
+			m_head.set(header::XORIG, h.h[header::XORIG]);
+		}
+		else
+		{
+			m_nSizeChecked=0;
+			m_head=h;
+		}
 		hint_length=atoofft(h.h[header::CONTENT_LENGTH], 0);
 		break;
 	}
 	case 206:
 	{
+
 		if(m_nSizeSeen<=0 && m_nRangeLimit<0)
 		{
 			// wtf? Cannot have requested partial content
@@ -392,7 +453,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		{
 			BOUNCE("500 Server reports unexpected range");
 		}
-	
+
 		m_nSizeChecked=myfrom;
 		
 		hint_start=myfrom;
@@ -402,6 +463,11 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		m_head.frontLine="HTTP/1.1 200 OK";
 		m_head.del(header::CONTENT_RANGE);
 		m_head.set(header::CONTENT_LENGTH, mylen);
+		m_head.set(header::XORIG, h.h[header::XORIG]);
+
+		// target opened before? close it, will reopen&seek later
+		if (bForcedRestart)
+			checkforceclose(m_filefd);
 
 		// special optimization; if "-1 trick" was used then maybe don't reopen that file for writing later
 		if(m_bCheckFreshness && pNextData && m_nSizeSeen == mylen && m_nSizeChecked == mylen-1)
@@ -420,16 +486,29 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 						m_nSizeChecked=mylen;
 					}
 				}
+				// XXX: optimize that, open as RW if possible and keep the file open for writing
 				forceclose(fd);
 			}
 		}
 		break;
 	}
 	case 416:
-		// that's always bad; it cannot have been complete before (the -1 trick)
+		// that's bad; it cannot have been complete before (the -1 trick)
+		// however, proxy servers with v3r4 cl3v3r caching strategy can cause that
+		// if if-mo-since is used and they don't like it, so attempt a retry in this case
+		if(m_nSizeChecked == 0)
+		{
+			USRDBG(0, "Peer denied to resume previous download (transient error) " << m_sPathRel );
+			m_nSizeSeen = 0;
+			return SuccessWithTransErrorFlag(false, true);
+		}
+		else
+		{
 		// -> kill cached file ASAP
-		m_bAllowStoreData=false;
-		SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
+			m_bAllowStoreData=false;
+			m_head.set(header::XORIG, h.h[header::XORIG]);
+			SETERRORKILLFILE("503 Server disagrees on file size, cleaning up");
+		}
 		break;
 // use the Location header and be sure the server does not mess around with Location and other codes
 	case 301:
@@ -439,10 +518,19 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		m_head.set(header::LOCATION, h.h[header::LOCATION]);
 		// fall-through
 	default:
+		m_head.type=header::ANSWER;
+		m_head.set(header::XORIG, h.h[header::XORIG]);
+		if(bForcedRestart)
+		{
+			// got a redirection from another mirror? or an error? cannot handle it yet.
+			// XXX: concept change required
+			USRDBG(0, "Cannot restart, HTTP code: " << serverStatus);
+			BOUNCE(h.getCodeMessage());
+		}
+
 		m_bAllowStoreData=false;
 		// have a clean header with just the error message
 		m_head.frontLine=h.frontLine;
-		m_head.type=header::ANSWER;
 		m_head.set(header::CONTENT_LENGTH, "0");
 		if(m_status>FIST_DLGOTHEAD)
 		{
@@ -456,7 +544,6 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 
 	if(m_bAllowStoreData)
 	{
-
 		// using adaptive Delete-Or-Replace-Or-CopyOnWrite strategy
 		
 		// First opening the file first to be sure that it can be written. Header storage is the critical point,
@@ -516,7 +603,11 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		if(m_nSizeChecked <= m_nSizeSeen)
 		{
 			if(0==ftruncate(m_filefd, m_nSizeChecked))
+      {
+#if ( (_POSIX_C_SOURCE >= 199309L || _XOPEN_SOURCE >= 500) && _POSIX_SYNCHRONIZED_IO > 0)
 				fdatasync(m_filefd);
+#endif
+      }
 			else
 				SETERRORKILLFILE("503 Cannot change file size");
 		}
@@ -580,29 +671,27 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 		if(FIST_COMPLETE == m_status)
 		{
 			LOG("already completed");
-			return true;
 		}
-
-		m_status=FIST_COMPLETE;
-
-		if(acfg::debug&LOG_MORE)
-			aclog::misc(tSS()<<"Download of "<<m_sPathRel<<" finished");
-		
-		// we are done! Fix header from chunked transfers?
-		if (m_filefd>=0 && ! m_head.h[header::CONTENT_LENGTH])
+		else
 		{
-			m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
-			m_head.StoreToFile(m_sPathAbs+".head");
+			m_status = FIST_COMPLETE;
+
+			if (acfg::debug & LOG_MORE)
+				aclog::misc(tSS() << "Download of " << m_sPathRel << " finished");
+
+			// we are done! Fix header from chunked transfers?
+			if (m_filefd >= 0 && !m_head.h[header::CONTENT_LENGTH])
+			{
+				m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
+				m_head.StoreToFile(m_sPathAbs + ".head");
+			}
 		}
 	}
 	else
 	{
 		m_status = FIST_DLRECEIVING;
 
-		if (!m_bAllowStoreData)
-			return true;
-
-		if (m_filefd>=0)
+		if (m_bAllowStoreData && m_filefd>=0)
 		{
 			while(size>0)
 			{
@@ -611,16 +700,23 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 				{
 					if(EINTR==errno || EAGAIN==errno)
 						continue;
-					aclog::errnoFmter efmt("503 ");
-					BOUNCE(efmt.c_str());
+					aclog::errnoFmter efmt("HTTP/1.1 503 ");
+					m_head.frontLine = efmt;
+					m_status=FIST_LOCALERROR;
+					_LogWithErrno(efmt.c_str(), m_sPathRel);
+					return false;
 				}
 				m_nSizeChecked+=r;
 				size-=r;
 				data+=r;
 			}
-			
 		}
 	}
+
+	// needs to remember the good size, just in case the DL is resumed (restarted in hot state)
+	if(m_nSizeSeen < m_nSizeChecked)
+		m_nSizeSeen = m_nSizeChecked;
+
 	return true;
 }
 
@@ -740,8 +836,8 @@ void fileitem::UnregOneASAP(tFileItemPtr pItem)
 	// now the item's refCount is by one higher then it should be, set the flag instead of --
 	entry->second.m_bNeedsDelayedExpiration=true;
 
-// this will work as long as the delay is equal for all
-	m_nEarliestExpiration=pItem->m_nTimeExpireAt;
+	m_nEarliestExpiration = MYSTD::min(pItem->m_nTimeExpireAt, m_nEarliestExpiration);
+	g_victor.RunLatestAt(m_nEarliestExpiration);
 
 }
 
@@ -758,7 +854,7 @@ time_t fileitem::DoDelayedUnregAndCheck()
 	if(now<m_nEarliestExpiration) // unreachable anyhow
 		return m_nEarliestExpiration;
 
-	m_nEarliestExpiration = cleaner::never; // the correct time will be found soon
+	m_nEarliestExpiration = g_victor.never(); // the correct time will be found below
 
 	for(it=mapItems.begin(); it!=mapItems.end();)
 	{
