@@ -20,7 +20,7 @@ using namespace MYSTD;
 
 #include <errno.h>
 
-// XXX no longer needed after debug logging overhaul
+// XXX no longer needed after debug logging overhaul?
 #define REPLEVEL (m_type==rechecks::FILE_INDEX ? 4 : (m_type==rechecks::FILE_PKG ? 5 : SPAMLEVEL))
 
 #if 0 // defined(DEBUG)
@@ -70,7 +70,8 @@ public:
 		}
 	};
 	virtual int GetFileFd() { return SPECIAL_FD; }; // something, don't care for now
-	virtual bool DownloadStartedStoreHeader(const header & h, const char *)
+	virtual SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header & h, const char *,
+			bool)
 	{
 		setLockGuard;
 		m_head=h;
@@ -183,7 +184,7 @@ public:
 		m_head.set(header::CONTENT_LENGTH, m_nSizeChecked);
 	}
 	// never used to store data
-	bool DownloadStartedStoreHeader(const header &, const char *) {return false;};
+	SuccessWithTransErrorFlag DownloadStartedStoreHeader(const header &, const char *, bool) {return false;};
 	bool StoreFileData(const char *, unsigned int) {return false;};
 };
 
@@ -237,17 +238,18 @@ job::job(header *h, con *pParent) :
 	m_filefd(-1),
 	m_pParentCon(pParent),
 	m_bChunkMode(CHUNKDEFAULT),
+	m_bClientWants2Close(false),
 	m_bIsHttp11(false),
 	m_bNoDownloadStarted(false),
 	m_state(STATE_SEND_MAIN_HEAD),
 	m_backstate(STATE_TODISCON),
 	m_pReqHead(h),
 	m_nSendPos(0),
-	m_nCurrentRangeLast(MAX_VAL(off_t)-1), // range size should never overflow
+	m_nCurrentRangeLast(MAX_VAL(off_t)-1),
 	m_nAllDataCount(0),
 	m_nChunkRemainingBytes(0),
-	m_type(FILE_INVALID),
-	m_nRangeFrom(-1), m_nRangeTo(-1),
+	m_type(rechecks::FILE_INVALID),
+	m_nReqRangeFrom(-1), m_nReqRangeTo(-1),
 	m_nRedirRemaining(acfg::redirmax)
 {
 	LOGSTART2("job::job", "job creating, " << m_pReqHead->frontLine << " and this: " << uintptr_t(this));
@@ -475,16 +477,16 @@ inline bool job::ParseRange()
 	if (pRange)
 	{
 		int nRangeItems = sscanf(pRange, "bytes=" OFF_T_FMT
-		"-" OFF_T_FMT, &m_nRangeFrom, &m_nRangeTo);
+		"-" OFF_T_FMT, &m_nReqRangeFrom, &m_nReqRangeTo);
 		// working around bad (old curl style) requests
 		if (nRangeItems <= 0)
 		{
 			nRangeItems = sscanf(pRange, "bytes "
-			OFF_T_FMT "-" OFF_T_FMT, &m_nRangeFrom, &m_nRangeTo);
+			OFF_T_FMT "-" OFF_T_FMT, &m_nReqRangeFrom, &m_nReqRangeTo);
 		}
 
 		if (nRangeItems < 1) // weird...
-			m_nRangeFrom = m_nRangeTo = -2;
+			m_nReqRangeFrom = m_nReqRangeTo = -2;
 		else
 			return true;
 	}
@@ -565,7 +567,7 @@ void job::PrepareDownload() {
 		m_bClientWants2Close = 0==strncasecmp(m_pReqHead->h[header::CONNECTION], "close", 5);
 
     // "clever" file system browsing attempt?
-	if(stmiss != sRawUriPath.find("..")
+	if(rechecks::Match(sRawUriPath, rechecks::NASTY_PATH)
 			|| stmiss != sRawUriPath.find("/_actmp")
 			|| startsWithSz(sRawUriPath, "/_"))
 		goto report_notallowed;
@@ -627,11 +629,13 @@ void job::PrepareDownload() {
 			}
 		}
 
+		using namespace rechecks;
+
 		for(tStrMap::const_iterator it=acfg::localdirs.find(tUrl.sHost);
 				it != acfg::localdirs.end() ; )
 		{
 			string realSubPath;
-			if(! UrlDecode(tUrl.sPath, realSubPath) || realSubPath.find("..") != stmiss)
+			if(! UrlDecode(tUrl.sPath, realSubPath) || Match(realSubPath, NASTY_PATH))
 				goto report_invpath; // what the...
 			HandleLocalDownload(sRawUriPath, it->second, realSubPath);
 			return;
@@ -645,9 +649,10 @@ void job::PrepareDownload() {
 			return;
 		}
 
-		m_type = rechecks::GetFiletype(tUrl.sPath);
+		m_type = GetFiletype(tUrl.sPath);
 
-		if ( m_type == FILE_INVALID ) goto report_notallowed;
+		if ( m_type == FILE_INVALID )
+			goto report_notallowed;
 		
 		// got something valid, has type now, trace it
 		USRDBG(REPLEVEL, "Processing new job, "<<m_pReqHead->frontLine);
@@ -660,7 +665,7 @@ void job::PrepareDownload() {
 		else
 			m_sFileLoc=tUrl.sHost+tUrl.sPath;
 
-		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==FILE_INDEX);
+		bForceFreshnessChecks = ( ! acfg::offlinemode && m_type == FILE_INDEX);
 
     	m_pItem=fileitem::GetRegisteredFileItem(m_sFileLoc);
 
@@ -690,10 +695,10 @@ void job::PrepareDownload() {
 	// early optimization for requests which want a defined file part which we already have
 	// no matter whether the file is complete or not
 	// handles different cases: GET with range, HEAD with range, HEAD with no range
-	if((m_nRangeFrom>=0 && m_nRangeTo>=0) || (m_pReqHead->type==header::HEAD && 0!=(m_nRangeTo=-1)))
+	if((m_nReqRangeFrom>=0 && m_nReqRangeTo>=0) || (m_pReqHead->type==header::HEAD && 0!=(m_nReqRangeTo=-1)))
 	{
 		lockguard g(*m_pItem);
-		if(m_pItem->CheckUsableRange_unlocked(m_nRangeTo))
+		if(m_pItem->CheckUsableRange_unlocked(m_nReqRangeTo))
 		{
 			LOG("Got a partial request for incomplete download; range is available");
 			m_bNoDownloadStarted=true;
@@ -807,7 +812,7 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 		{
 			fistate=m_pItem->GetStatusUnlocked(nGoodDataSize);
 			
-			dbgline;
+			LOG(fistate);
 			if (fistate>=FIST_LOCALERROR)
 			{
 				const header &h = m_pItem->GetHeaderUnlocked();
@@ -832,9 +837,8 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 				fistate = FIST_DLRECEIVING;
 				break;
 			}
-
 			// or wait for the dl source to get data at the position we need to start from
-			dbgline;
+			LOG(fistate << m_nSendPos << nGoodDataSize);
 			if(fistate==FIST_COMPLETE || (m_nSendPos < nGoodDataSize && fistate>=FIST_DLGOTHEAD))
 				break;
 			
@@ -858,13 +862,6 @@ job::eJobResult job::SendData(int confd, int nAllJobCount)
 
 	for(;;) // left by returning
 	{
-
-		/*
-		USRDBG(10,  m_sFileLoc << " : to state " << m_state
-				<< " and bs: " << m_backstate <<
-				" with fistate " << fistate << " sendpos: " << m_nSendPos
-				<< " goodsize: " << nGoodDataSize);
-*/
 		MYTRY // for bad_alloc in members
 		{
 			switch(m_state)
@@ -1030,7 +1027,10 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 	LOGSTART("job::BuildAndEnqueHeader");
 
 	if(respHead.type != header::ANSWER)
+	{
+		LOG(respHead.ToString());
 		return "500 Rotten Data";
+	}
 
 	// make sure that header has consistent state and there is data to send which is expected by the client
 	int httpstatus = respHead.getStatus();
@@ -1073,7 +1073,7 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			const char *pLastMo = respHead.h[header::LAST_MODIFIED];
 
 			// consider contents "fresh" for non-volatile data, or when "our" special client is there, or the client simply doesn't care
-			bool bFreshnessForced = (m_type != FILE_INDEX
+			bool bFreshnessForced = (m_type != rechecks::FILE_INDEX
 				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
 			// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
@@ -1091,10 +1091,10 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 			{
 				off_t nContLen=atoofft(respHead.h[header::CONTENT_LENGTH]);
 
-				if(m_nRangeFrom >=0)
+				if(m_nReqRangeFrom >=0)
 				{
-					if(m_nRangeTo<0 || m_nRangeTo>=nContLen) // open-end? set the end to file length. Also when request range would be too large
-						m_nRangeTo=nContLen-1;
+					if(m_nReqRangeTo<0 || m_nReqRangeTo>=nContLen) // open-end? set the end to file length. Also when request range would be too large
+						m_nReqRangeTo=nContLen-1;
 
 					/*
 					 * make sure that our client doesn't just hang here while the download thread is
@@ -1106,7 +1106,7 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 							|| (
 									fistate >= FIST_DLGOTHEAD
 									&& fistate <= FIST_COMPLETE
-									&& nGooddataSize>=m_nRangeFrom
+									&& nGooddataSize>=m_nReqRangeFrom
 								)
 							// don't care, found special hint from acngfs (kludge...)
 							|| m_pReqHead->h[header::ACNGFSMARK]
@@ -1116,11 +1116,11 @@ inline const char * job::BuildAndEnqueHeader(const FiStatus &fistate,
 						USRDBG(20, "request range problem: ");
 
 						// detect errors, out-of-range case
-						if(m_nRangeFrom>=nContLen || m_nRangeTo<m_nRangeFrom)
+						if(m_nReqRangeFrom>=nContLen || m_nReqRangeTo<m_nReqRangeFrom)
 							return "416 Requested Range Not Satisfiable";
 
-						m_nSendPos = m_nRangeFrom;
-						m_nCurrentRangeLast = m_nRangeTo;
+						m_nSendPos = m_nReqRangeFrom;
+						m_nCurrentRangeLast = m_nReqRangeTo;
 						// replace with partial-response header
 						sb.clear();
 						sb << "HTTP/1.1 206 Partial Response\r\nContent-Length: "
