@@ -311,21 +311,6 @@ string tHttpUrl::ToURI() const
 { 
 	return string("http://") + (sPort.empty()? (sHost+sPath) : (sHost+":"+sPort+sPath));
 }
-//tHttpUrl::tHttpUrl() : pMisc(NULL) {};
-
-/*
-const MYSTD::string GetDateString() {
-    char buf[26];
-    struct tm tmp;
-    const time_t cur=time(NULL);
-    gmtime_r(&cur, &tmp);
-    asctime_r(&tmp, buf);
-    buf[24]=0x0;
-//    cout << "generated date: " << buf<<endl;
-    return buf;
-    
-}
-*/
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
 
