@@ -74,7 +74,7 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 
 		AddIFileCandidate(sPath.substr(CACHE_BASE_LEN));
 	}
-	else if(FILE_INVALID != rechecks::GetFiletype(sPath))
+	else if(rechecks::FILE_INVALID != rechecks::GetFiletype(sPath))
 	{
 		// get a fingerprint by checksumming if not already there from the fpr cache
 
