@@ -279,15 +279,17 @@ void tcpconnect::RecycleIdleConnection(tTcpHandlePtr & handle)
 	{
 		spareConPool.insert(make_pair(tHostHint(handle->GetHostname(), handle->GetPort()),
 				make_pair(handle, now)));
+
+#ifndef MINIBUILD
+		g_victor.RunLatestAt(now+TIME_SOCKET_EXPIRE_CLOSE);
+#endif
+
 	}
 #endif
 
 	handle.reset();
 }
 
-
-
-
 time_t tcpconnect::ExpireCache()
 {
 	lockguard __g(conPoolMx);
@@ -324,7 +326,7 @@ time_t tcpconnect::ExpireCache()
 			++it;
 	}
 
-	return spareConPool.empty() ? cleaner::never : GetTime()+TIME_SOCKET_EXPIRE_CLOSE/4+1;
+	return spareConPool.empty() ? g_victor.never() : GetTime()+TIME_SOCKET_EXPIRE_CLOSE/4+1;
 }
 
 void tcpconnect::KillLastFile()
